/**
 *
 * Copyright 2005-2006 The Apache Software Foundation
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package org.apache.activemq.transport;

import java.io.IOException;

import org.apache.activemq.command.Command;

/**
 * An asynchronous listener of commands
 *
 * @version $Revision$
 */
public interface TransportListener {
    
    /**
     * called to process a command
     * @param command
     */
    public void onCommand(Command command);
    /**
     * An unrecoverable exception has occured on the transport
     * @param error
     */
    public void onException(IOException error);
    
    /**
     * The transport has suffered an interuption from which it hopes to recover
     *
     */
    public void transportInterupted();
    
    
    /**
     * The transport has resumed after an interuption
     *
     */
    public void transportResumed();
    
}
